package com.zsc.edu.gateway.modules.iot.product.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.zsc.edu.gateway.framework.message.sse.SseConfig;
import com.zsc.edu.gateway.framework.mybatisplus.DataPermission;
import com.zsc.edu.gateway.modules.iot.product.dto.ProductDto;
import com.zsc.edu.gateway.modules.iot.product.entity.Product;
import com.zsc.edu.gateway.modules.iot.product.query.ProductQuery;
import com.zsc.edu.gateway.modules.iot.product.service.ProductService;
import com.zsc.edu.gateway.modules.iot.product.service.impl.ProductServiceImpl;
import com.zsc.edu.gateway.modules.iot.product.vo.ProductStatusVo;
import jakarta.annotation.Resource;
import lombok.AllArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.web.PageableDefault;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.mvc.method.annotation.SseEmitter;

import java.io.IOException;
import java.util.Collection;
import java.util.List;
import java.util.concurrent.ExecutorService;

/**
 * @author zhuang
 */
@AllArgsConstructor
@RestController
@RequestMapping("api/rest/product")
public class ProductController {

    private final ProductService service;

    @Resource
    SseConfig sseConfig;

    /**
     * 创建产品
     *
     * @param dto 创建的产品
     * @return 新的产品
     */
    @PostMapping
    @PreAuthorize("hasAuthority('iot:product:create')")
    public Product create(@RequestBody ProductDto dto) {
        return service.create(dto);
    }


    /**
     * 更新产品
     *
     * @return 更新后的产品
     */
    @PatchMapping("{id}")
    @PreAuthorize("hasAuthority('iot:product:update')")
    public Product update(@RequestBody ProductDto dto, @PathVariable("id") Long id) {
        return service.update(dto, id);
    }


    /**
     * 查询产品
     *
     * @param query    产品的查询表单
     * @param page  分页参数
     * @return Page<Device> 产品分页数据
     */
    @DataPermission
    @GetMapping
    @PreAuthorize("hasAuthority('iot:product:query')")
    public Page<Product> page(Page<Product> page, ProductQuery query) {
        return service.page(page, query.wrapper());
    }

    /**
     * 模糊查询名字，最多返回5条数据
     *
     * @param query 查询参数
     * @return 部门列表
     */
    @GetMapping("fuzzy")
    @PreAuthorize("hasAuthority('iot:product:query')")
    public List<Product> list(ProductQuery query) {
        return service.list(query);
    }

    /**
     * 删除产品
     *
     * @param id 产品编号
     * @return 被删除的产品
     */
    @DeleteMapping("{id}")
    @PreAuthorize("hasAuthority('iot:product:delete')")
    public Boolean delete(@PathVariable("id") Long id) {
        return service.delete(id);
    }

    /**
     * 产品
     * 查询详情
     *
     * @param id ID
     * @return 任务
     */
    @GetMapping("{id}")
    @PreAuthorize("hasAuthority('iot:product:query')")
    public Product detail(@PathVariable("id") Long id) {
        return service.detail(id);
    }

    /**
     * 查询产品各类信息
     */
    @GetMapping(value = "/sse/status", produces = MediaType.TEXT_EVENT_STREAM_VALUE)
    @DataPermission
    @PreAuthorize("hasAuthority('iot:product:query')")
    public SseEmitter status() {
        return sseConfig.sendSseEvents(service::status);
    }
}

